@@ -2,7 +2,8 @@
  * AT-SPI - Assistive Technology Service Provider Interface
  * (Gnome Accessibility Project; http://developer.gnome.org/projects/gap)
  *
- * Copyright 2001 Sun Microsystems Inc.
+ * Copyright 2001, 2002 Sun Microsystems Inc.,
+ * Copyright 2001, 2002 Ximian, Inc.
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Library General Public
@@ -26,7 +27,6 @@
 #include <libspi/Accessibility.h>
 #include "accessible.h"
 #include "listener.h"
-#include "desktop.h"
 
 void
 check_ev (CORBA_Environment *ev, char *desc)
@@ -50,6 +50,7 @@ main(int argc, char **argv)
         CORBA_short n_desktops;
         CORBA_long j;
         CORBA_long n_apps;
+	CORBA_string s;
         Accessibility_Desktop desktop;
         Accessibility_Application app;
         Accessibility_Registry registry;
@@ -62,7 +63,7 @@ main(int argc, char **argv)
             g_error ("Could not initialize Bonobo");
           }
 
-        obj_id = "OAFIID:Accessibility_Registry:proto0.1";
+        obj_id = "OAFIID:Accessibility_Registry:1.0";
 
         oclient = bonobo_activation_activate_from_id (obj_id, 0, NULL, &ev);
         if (ev._major != CORBA_NO_EXCEPTION) {
@@ -105,8 +106,9 @@ main(int argc, char **argv)
             for (i=0; i<n_desktops; ++i)
               {
                 desktop = Accessibility_Registry_getDesktop (registry, i, &ev);
-                fprintf (stderr, "desktop %d name: %s\n", i,
-                         Accessibility_Desktop__get_name (desktop, &ev));
+		s = Accessibility_Desktop__get_name (desktop, &ev);
+                fprintf (stderr, "desktop %d name: %s\n", i, s);
+	        CORBA_free (s);	 
                 check_ev (&ev, "desktop:name");
                 n_apps = Accessibility_Desktop__get_childCount (desktop, &ev);
                 check_ev (&ev, "desktop:childCount");
@@ -115,8 +117,9 @@ main(int argc, char **argv)
                   {
                     app = (Accessibility_Application) Accessibility_Desktop_getChildAtIndex (desktop, j, &ev);
                     check_ev (&ev, "desktop:getChildAtIndex");
-                    fprintf (stderr, "app %d name: %s\n", j,
-                             Accessibility_Application__get_name (app, &ev));
+                    s = Accessibility_Application__get_name (app, &ev);
+		    fprintf (stderr, "app %d name: %s\n", j, s);
+		    CORBA_free (s);
                     check_ev (&ev, "app:getName");
                   }
               }
